﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Analysis;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Analyzers {
  [Item("SymbolicDataAnalysisBottomUpDiversityAnalyzer", "A diversity analyzer based on the bottom-up similarity between trees.")]
  [StorableClass]
  public class SymbolicDataAnalysisBottomUpDiversityAnalyzer : SingleObjectivePopulationDiversityAnalyzer {
    private const string MatchConstantValuesParameterName = "MatchConstantValues";
    private const string MatchVariableWeightsParameterName = "MatchVariableWeights";

    public IValueParameter<BoolValue> MatchVariableWeightsParameter {
      get { return (IValueParameter<BoolValue>)Parameters[MatchVariableWeightsParameterName]; }
    }

    public IValueParameter<BoolValue> MatchConstantValuesParameter {
      get { return (IValueParameter<BoolValue>)Parameters[MatchConstantValuesParameterName]; }
    }

    public bool MatchConstantValues {
      get { return MatchConstantValuesParameter.Value.Value; }
      set {
        MatchConstantValuesParameter.Value.Value = value;
        ((BottomUpSimilarityCalculator)SimilarityCalculator).MatchConstantValues = value;
      }
    }

    public bool MatchVariableWeights {
      get { return MatchVariableWeightsParameter.Value.Value; }
      set {
        MatchVariableWeightsParameter.Value.Value = value;
        ((BottomUpSimilarityCalculator)SimilarityCalculator).MatchVariableWeights = value;
      }
    }

    [StorableConstructor]
    protected SymbolicDataAnalysisBottomUpDiversityAnalyzer(bool deserializing) : base(deserializing) { }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      if (!Parameters.ContainsKey(MatchConstantValuesParameterName))
        Parameters.Add(new ValueParameter<BoolValue>(MatchConstantValuesParameterName, "Specifies if the similarity calculator should match constant values.", new BoolValue(true)));
      if (!Parameters.ContainsKey(MatchVariableWeightsParameterName))
        Parameters.Add(new ValueParameter<BoolValue>(MatchVariableWeightsParameterName, "Specifies if the similarity calculator should match variable weights", new BoolValue(true)));
    }

    public SymbolicDataAnalysisBottomUpDiversityAnalyzer() {
      SimilarityCalculator = new BottomUpSimilarityCalculator { SolutionVariableName = "SymbolicExpressionTree" };
      Parameters.Add(new ValueParameter<BoolValue>(MatchConstantValuesParameterName, "Specifies if the similarity calculator should match constant values.", new BoolValue(true)));
      Parameters.Add(new ValueParameter<BoolValue>(MatchVariableWeightsParameterName, "Specifies if the similarity calculator should match variable weights", new BoolValue(true)));
    }
  }
}
